//-------------------------------------------------------------------------------------------------
/// \file	BasicHwShaderBehavior.h
/// \date	6-11-04
/// \author	Rob Bateman
/// \brief	Up until now, we have had a couple of problems with our node. For example, if within
/// 		the hypershade we drag the HW shader onto say a lambert or phong, Maya will not make
/// 		a connection to the node.
/// 			Essentially Maya will not know how to connect our nodes to other nodes, so we have
/// 		to provide maya with that information. We do this by simply providing Maya with our own
/// 		class derived from MPxDragAndDropBehavior.
///
/// 			Primarily we have to tell maya when to use this behaviour class by overloading the
///			shouldBeUsedFor function. In addition we can then specify the following functions to
/// 		connect the nodes and attributes correctly....
///
/// 					connectNodeToNode
/// 					connectNodeToAttr
/// 					connectAttrToNode
/// 					connectAttrToAttr
///
//-------------------------------------------------------------------------------------------------

#ifndef __BASIC_HW_SHADER_BEHAVIOUR__H__
#define __BASIC_HW_SHADER_BEHAVIOUR__H__

	#ifdef WIN32
		#define NT_PLUGIN
		#pragma once
	#endif

	#include <maya/MPxDragAndDropBehavior.h>

	/// \brief	Defines the drag and drop behaviour of the nodes within the plugin. By providing
	/// 		this class we are allowing the node to work as we intend within the hypergraph
	/// 		and hypershade.
	///
	class BasicHwShaderBehavior
		: public MPxDragAndDropBehavior
	{
	public:

		/// \brief	We MUST overload this function to tell maya which connections this
		/// 		class will handle. ie, does it support connections from one node
		/// 		type to another?
		/// \param	srcNode		-	the input node
		/// \param	dstNode		-	the output node
		/// \param	srcPlug		-	the input attribute
		/// \param	dstPlug		-	the output attribute
		/// \return	true if this behaviour class handles this node connection.
		/// \note	Since this class can handle connections between nodes, you may well find
		/// 		that the plugs are NULL. To be safe, make sure you test the plugs with
		/// 		the member function isNull() before doing anything with them!
		///
		virtual bool shouldBeUsedFor( MObject&, MObject&, MPlug&, MPlug& );



		/// \brief	Overide this function to connect nodes to nodes
		/// \param	srcNode		-	the input node
		/// \param	dstNode		-	the output node
		/// \param	force		-	any existing connections should be broken to make
		/// 							this connection occur.
		///	\return	MS::kSuccess or MS::kFailure
		///
		virtual MStatus connectNodeToNode( MObject&, MObject&, bool );

		/// \brief	Overide this function to connect nodes to attributes
		/// \param	srcNode		-	the input node
		/// \param	dstPlug		-	the output attribute
		/// \param	force		-	any existing connections should be broken to make
		/// 							this connection occur.
		///	\return	MS::kSuccess or MS::kFailure
		///
		virtual MStatus connectNodeToAttr( MObject&, MPlug&, bool );

		/// \brief	Overide this function to connect attributes to nodes
		/// \param	srcPlug		-	the input attribute
		/// \param	dstNode		-	the output node
		/// \param	force		-	any existing connections should be broken to make
		/// 							this connection occur.
		///	\return	MS::kSuccess or MS::kFailure
		///
		virtual MStatus connectAttrToNode( MPlug&, MObject&, bool );

		/// \brief	Overide this function to connect attributes to attributes
		/// \param	srcPlug		-	the input attribute
		/// \param	dstPlug		-	the output attribute
		/// \param	force		-	any existing connections should be broken to make
		/// 							this connection occur.
		///	\return	MS::kSuccess or MS::kFailure
		///
		virtual MStatus connectAttrToAttr( MPlug&, MPlug&, bool );

	public:

		/// \brief	Used to return a new instance of this behaviour node to Maya
		/// \return the new behaviour instance
		///
		static void *creator();

	};


#endif
